/**
 * OW2 Util
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AbstractRmicMojo.java 2483 2008-02-20 09:02:57Z benoitf $
 * --------------------------------------------------------------------------
 */

/*
 * Copyright 2005-2006 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.maven.plugin.rmic;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.DependencyResolutionRequiredException;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.apache.tools.ant.DefaultLogger;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Target;
import org.apache.tools.ant.taskdefs.Rmic;
import org.apache.tools.ant.taskdefs.Rmic.ImplementationSpecificArgument;
import org.apache.tools.ant.types.Path;
import org.codehaus.plexus.util.StringUtils;

/**
 * Maven Rmic Mojo.
 * @author Gael Lalire
 */
public abstract class AbstractRmicMojo extends AbstractMojo {

    /**
     * @parameter expression="${project}"
     * @required
     * @readonly
     */
    private MavenProject mavenProject;

    /**
     * This folder is added to the list of those folders containing source to be
     * compiled. Use this if your ant script generates source code.
     *
     * @parameter expression="${sourceRoot}"
     */
    private File sourceRoot;

    /**
     * This folder is added to the list of those folders containing source to be
     * compiled for testing. Use this if your ant script generates test source
     * code.
     *
     * @parameter expression="${testSourceRoot}"
     */
    private File testSourceRoot;

    /**
     * The plugin dependencies.
     *
     * @parameter expression="${plugin.artifacts}"
     * @required
     * @readonly
     */
    private List<Artifact> artifacts;

    /* BEGIN APACHE ANT RMIC (these variables are specific to rmic plugin) */

    /**
     * Compiler.
     *
     * @parameter expression="${compiler}"
     * @readonly
     */
    private String compiler;

    /**
     * Ant rmic task param.
     * @parameter expression="${classname}"
     * @required
     */
    private String classname;

    /**
     * Ant rmic task param.
     * @parameter expression="${stubVersion}"
     */
    private String stubVersion;

    /**
     * Ant rmic task param.
     * @parameter expression="${verify}" default-value="false"
     */
    private boolean verify;

    /**
     * Ant rmic task param.
     * @parameter expression="${filtering}" default-value="false"
     */
    private boolean filtering;

    /**
     * Ant rmic task param.
     * @parameter expression="${iiop}" default-value="false"
     */
    private boolean iiop;

    /**
     * Ant rmic task param.
     * @parameter expression="${iiopOpts}"
     */
    private String iiopOpts;

    /**
     * Ant rmic task param.
     * @parameter expression="${idl}" default-value="false"
     */
    private boolean idl;

    /**
     * Ant rmic task param.
     * @parameter expression="${idlOpts}"
     */
    private String idlOpts;

    /**
     * Ant rmic task param.
     * @parameter expression="${debug}" default-value="false"
     */
    private boolean debug;

    // TODO extDir create a pojo for creation of a path
    // IGNORE private Path compileClasspath;
    // IGNORE private Path extDirs;
    // IGNORE private boolean includeAntRuntime = true;
    // IGNORE private boolean includeJavaRuntime = false;
    // IGNORE private Vector compileList = new Vector();
    // IGNORE private ClassLoader loader = null;
    // IGNORE private FacadeTaskHelper facade;

    /* END APACHE ANT RMIC */

    /**
     * Compiler args.
     * @parameter expression="${compilerarg}"
     */
    private String compilerarg;

    /**
     * Create ant task with properties of the mojo.
     * @return Ant target which contain rmic Task
     * @throws DependencyResolutionRequiredException when getCompileClasspathElements failed
     */
    public Target createAntTask() throws DependencyResolutionRequiredException {
        Target target = new Target();
        Project antProject = new Project();
        target.setName("");
        antProject.setName("DummyProject");
        target.setProject(antProject);
        antProject.addTarget(target);

        Rmic antRmic = new Rmic();
        antRmic.setTaskName("rmic");
        antRmic.setProject(antProject);
        antProject.init();

        Path p = new Path(antProject);
        p.setPath(StringUtils.join(mavenProject.getCompileClasspathElements().iterator(), File.pathSeparator));
        antRmic.setClasspath(p);


        antRmic.setBase(getBaseDir());
        antRmic.setClassname(classname);
        if (compiler == null || compiler.equalsIgnoreCase("sun")) {
            antRmic.setCompiler("org.ow2.util.maven.plugin.rmic.SunRmicProxy");
        } else {
            antRmic.setCompiler(compiler);
        }
        antRmic.setSourceBase(sourceRoot);
        antRmic.setStubVersion(stubVersion);
        antRmic.setVerify(verify);
        antRmic.setFiltering(filtering);
        antRmic.setIiop(iiop);
        antRmic.setIiopopts(iiopOpts);
        antRmic.setIdl(idl);
        antRmic.setIdlopts(idlOpts);
        antRmic.setDebug(debug);
        if (compilerarg != null) {
            ImplementationSpecificArgument implementationSpecificArgument = antRmic.createCompilerArg();
            implementationSpecificArgument.setValue(compilerarg);
        }
        target.addTask(antRmic);
        return target;
    }

    /**
     * Execute method of the mojo.
     * @throws MojoExecutionException on error
     */
    public void execute() throws MojoExecutionException {
        try {
            executeTasks(createAntTask());
        } catch (DependencyResolutionRequiredException e) {
            throw new MojoExecutionException("DependencyResolutionRequiredException : ", e);
        }
        if (sourceRoot != null) {
            getLog().info("Registering compile source root " + sourceRoot);
            mavenProject.addCompileSourceRoot(sourceRoot.toString());
        }

        if (testSourceRoot != null) {
            getLog().info("Registering compile test source root " + testSourceRoot);
            mavenProject.addTestCompileSourceRoot(testSourceRoot.toString());
        }
    }

    /**
     * Execute Tasks.
     * @param antTasks the target to execute
     * @throws MojoExecutionException on error
     */
    private void executeTasks(final Target antTasks) throws MojoExecutionException {
        try {
            // TODO refactor - place the manipulation of the expressionEvaluator
            // into a separated class.
            // ExpressionEvaluator exprEvaluator = (ExpressionEvaluator)
            // antTasks.getProject().getReference(AntTargetConverter.MAVEN_EXPRESSION_EVALUATOR_ID);

            Project antProject = antTasks.getProject();

            // PropertyHelper propertyHelper = PropertyHelper.getPropertyHelper(
            // antProject );
            // propertyHelper.setNext( new AntPropertyHelper( exprEvaluator,
            // getLog() ) );

            DefaultLogger antLogger = new DefaultLogger();
            antLogger.setOutputPrintStream(System.out);
            antLogger.setErrorPrintStream(System.err);
            antLogger.setMessageOutputLevel(Project.MSG_INFO);

            antProject.addBuildListener(antLogger);
            antProject.setBaseDir(mavenProject.getBasedir());

            Path p = new Path(antProject);
            p.setPath(StringUtils.join(mavenProject.getArtifacts().iterator(), File.pathSeparator));
            antProject.addReference("maven.dependency.classpath", p);
            p = new Path(antProject);
            p.setPath(StringUtils.join(mavenProject.getCompileClasspathElements().iterator(), File.pathSeparator));
            antProject.addReference("maven.compile.classpath", p);
            p = new Path(antProject);
            p.setPath(StringUtils.join(mavenProject.getRuntimeClasspathElements().iterator(), File.pathSeparator));
            antProject.addReference("maven.runtime.classpath", p);
            p = new Path(antProject);
            p.setPath(StringUtils.join(mavenProject.getTestClasspathElements().iterator(), File.pathSeparator));
            antProject.addReference("maven.test.classpath", p);
            /* set maven.plugin.classpath with plugin dependencies */
            List<String> list = new ArrayList<String>(artifacts.size());
            for (Artifact a : artifacts) {
                File file = a.getFile();
                if (file == null) {
                    throw new DependencyResolutionRequiredException(a);
                }
                list.add(file.getPath());
            }
            p = new Path(antProject);
            p.setPath(StringUtils.join(list.iterator(), File.pathSeparator));
            antProject.addReference("maven.plugin.classpath", p);
            antTasks.execute();
        } catch (Exception e) {
            throw new MojoExecutionException("Error executing ant tasks ", e);
        }
    }

    /**
     * Return the location to get the compiled files.
     * @return the location to get the compiled files
     */
    public abstract File getBaseDir();

}
